/*
 * HeightProfile.java
 *
 * Created on 26  2004 ., 12:00
 */

package com.fn.BikersLog;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class HeightProfile {

    private List vertices = new ArrayList();
    private boolean sorted = false;
    private double minHeight;
    private double maxHeight;
    private List listeners = new LinkedList();
    
    /** Creates a new instance of HeightProfile */
    public HeightProfile() {
    }
    
    public HeightProfile(Element tag) throws Exception {
        if (null == tag)
            return;
        load(tag);
    }

    public HeightProfile(File file) throws Exception {
        InputStream is = new BufferedInputStream(new FileInputStream(file));
        Document document = DOMUtil.parse(is);
        Element root = document.getDocumentElement();
        if (! root.getNodeName().equals("heightprofile"))
            throw new Exception("Invalid document");
        load(root);
    }
    
    public HeightProfile(HeightProfile toClone) {
        for (Iterator i = toClone.vertices.iterator(); i.hasNext(); )
            vertices.add(new HeightVertex((HeightVertex)i.next()));
    }
    
    private void load(Element tag) throws Exception {
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); )
            vertices.add(new HeightVertex(i.next()));
    }
    
    private void save(Document doc, Element parent) {
        for (Iterator i = vertices.iterator(); i.hasNext(); )
            ((HeightVertex)i.next()).save(doc, parent);
    }
    
    public void save(Document doc, Element parent, String name) {
        Element tag = DOMUtil.addElement(doc, parent, name);
        save(doc, tag);
    }
    
    public void save(File file) throws Exception {
        Document document = DOMUtil.createDocument();
        Element root = (Element)document.createElement("heightprofile");
        root.setAttribute("major", "1");
        root.setAttribute("minor", "0");
        document.appendChild(root);
        save(document, root);
        document.getDocumentElement().normalize();
        OutputStream os = new BufferedOutputStream(new FileOutputStream(file));
        DOMUtil.writeXmlToFile(os, document);
    }

    public boolean isEmpty() {
        return vertices.size() < 2;
    }
    
    public int getSize() {
        return vertices.size();
    }
    
    public void setFlat(double length) {
        HeightVertex v1 = null;
        HeightVertex v2 = null;
        if (vertices.size() > 0) {
            v1 = (HeightVertex)vertices.get(0);
            if (vertices.size() > 1)
                v2 = (HeightVertex)vertices.get(1);
        }
        vertices.clear();
        if (null == v1)
            v1 = new HeightVertex();
        v1.setPosition(0.0);
        v1.setHeight(-10.0);
        if (null == v2)
            v2 = new HeightVertex();
        v2.setPosition(length);
        v2.setHeight(0.0);
        vertices.add(v1);
        vertices.add(v2);
        sorted = false;
        fireDataChanged();
    }
    
    public HeightVertex getVertix(int index) {
        return (HeightVertex)vertices.get(index);
    }
    
    public void order() {
        Collections.sort(vertices);
        minHeight = 0.0;
        maxHeight = 0.0;
        boolean first = true;
        for (Iterator i = vertices.iterator(); i.hasNext(); ) {
            HeightVertex v = (HeightVertex)i.next();
            double h = v.getHeight();
            if (first) {
                minHeight = h;
                maxHeight = h;
                first = false;
            } else {
                if (minHeight > h)
                    minHeight = h;
                if (maxHeight < h)
                    maxHeight = h;
            }
        }
        sorted = true;
    }
    
    public void markAsUnordered() {
        sorted = false;
        fireDataChanged();
    }
    
    public double getLength() {
        if (isEmpty())
            return 0.0;
        
        if (! sorted)
            order();
        
        int cnt = vertices.size();
        return ((HeightVertex)vertices.get(cnt - 1)).getPosition();
    }
    
    public double getMinHeight() {
        if (! sorted)
            order();
        return minHeight;
    }
    
    public double getMaxHeight() {
        if (! sorted)
            order();
        return maxHeight;
    }
    
    private void fireDataChanged() {
        for (Iterator i = listeners.iterator(); i.hasNext(); )
            ((HeightProfileListener)i.next()).onProfileChanged(this);
    }
    
    public void addListener(HeightProfileListener listener) {
        listeners.add(listener);
    }
    
    public void removeListener(HeightProfileListener listener) {
        listeners.remove(listener);
    }
    
    public void clear() {
        vertices.clear();
        fireDataChanged();
    }
    
    public void addVertex(double pos, double height) {
        vertices.add(new HeightVertex(pos, height));
        sorted = false;
        fireDataChanged();
    }
    
    public void deleteVertex(int no) {
        vertices.remove(no);
        sorted = false;
        fireDataChanged();
    }

    public double getClimbPercents() {
        double length = getLength();
        if (0.0 == length)
            return 0.0;
        double totalClimb = 0.0;
        HeightVertex prevV = null;
        for (Iterator i = vertices.iterator(); i.hasNext(); ) {
            HeightVertex v = (HeightVertex)i.next();
            if (prevV != null)
                if (prevV.getHeight() < v.getHeight())
                    totalClimb += v.getPosition() - prevV.getPosition();
            prevV = v;
        }
        return length / 100.0 * totalClimb;
    }
    
    public double getElevation() {
        if (isEmpty())
            return 0.0;
        HeightVertex first = (HeightVertex)vertices.get(0);
        HeightVertex last = (HeightVertex)vertices.get(vertices.size() - 1);
        return last.getHeight() - first.getHeight();
    }
}
